"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TISHomebridge = void 0;
const settings_1 = require("homebridge-tiscontrol-wired/app/settings");
const RelaySwitch = require("homebridge-tiscontrol-wired/app/switch");
const Relaydimmer = require("homebridge-tiscontrol-wired/app/dimmer");
const Dimmerfan = require("homebridge-tiscontrol-wired/app/fan");
const Appliance = require("homebridge-tiscontrol-wired/app/appliance");
const CurtainMotor = require("homebridge-tiscontrol-wired/app/curtain_motor");
const Thermostat = require("homebridge-tiscontrol-wired/app/thermostat");
const IR_TV = require("homebridge-tiscontrol-wired/app/tv");
const RGB = require("homebridge-tiscontrol-wired/app/rgb");
const Universal_Switch = require("homebridge-tiscontrol-wired/app/universal_switch");
const RelayToCurtain = require("homebridge-tiscontrol-wired/app/relay_to_curtain");
const FloorHeater = require("homebridge-tiscontrol-wired/app/floorheater");
const BeeCurtain = require("homebridge-tiscontrol-wired/app/bee_curtain");
const  helper = require("homebridge-tiscontrol-wired/app/helper");
const  Extra = require("homebridge-tiscontrol-wired/app/extra");
const Security = require("homebridge-tiscontrol-wired/app/security_sensor");

// feedback reader 

const crc_calculator = require("homebridge-tiscontrol-wired/app/crc/crc16");
const send_to_server = require("homebridge-tiscontrol-wired/app/server");




const dgram = require('dgram');

// status 


class TISHomebridge {

    constructor(log, config, api) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.Service = this.api.hap.Service;
        this.Characteristic = this.api.hap.Characteristic;
        this.accessories = [];
        this.log.debug('Finished initializing platform:');
        this.api.on('didFinishLaunching', () => {
            log.debug('Executed didFinishLaunching callback');
            this.discoverDevices();
            this.UDPDevices();
        });
    }
    configureAccessory(accessory) {
        this.log.debug('Loading accessory from cache:', accessory.displayName);
        this.accessories.push(accessory);

    }
    discoverDevices() {
        

        if(this.config.devices && this.config.devices.length !==0){

         for (const device of this.config.devices) {

            const device_name = device.device_name;
            const subnet_id = device.subnet_id;
            const device_id = device.device_id;
            const channel_id = device.channel_id;
            const device_type = device.device_type;
            const ip_address=this.config.ip_address
            
            switch (device_type) {

                case 'switch':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                   // this.log.debug(' switch uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new RelaySwitch.RelaySwitch(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new RelaySwitch.RelaySwitch(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'dimmer':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    //this.log.debug('dimmer uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Relaydimmer.RelayDimmer(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,25)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Relaydimmer.RelayDimmer(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,25);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'fan':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('fan uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Dimmerfan.DimmerFan(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,25)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Dimmerfan.DimmerFan(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,25);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'appliances':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('appliance uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Appliance.Appliance(this, existingAccessory,  ip_address,device_name,subnet_id,device_id,channel_id,false,null)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Appliance.Appliance(this, accessory,  ip_address,device_name,subnet_id,device_id,channel_id,false,null);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'curtain_motor':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('curtain_motor uuid is '+uuid);
                    const send_option=device.send_option
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new CurtainMotor.CurtainMotor(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,send_option)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new CurtainMotor.CurtainMotor(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,send_option);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'relay_to_curtain':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('relay to curtain uuid is '+uuid);
                    const close_id=device.close_id
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new RelayToCurtain.RelayToCurtain(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,close_id)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new RelayToCurtain.RelayToCurtain(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,close_id);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'bee_curtain':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('curtain_motor uuid is '+uuid);
                    
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new BeeCurtain.BeeCurtain(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new BeeCurtain.BeeCurtain(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'thermostat':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('thermostat uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Thermostat.Thermostat(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,device.scale)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Thermostat.Thermostat(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,device.scale);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;
                case 'floor_heater':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('floor heater uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new FloorHeater.FloorHeater(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,device.scale)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new FloorHeater.FloorHeater(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,null,device.scale);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'tv':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new IR_TV.IR_TV(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new IR_TV.IR_TV(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id);
                        
                        accessory.category = this.api.hap.Categories.TELEVISION;

                       
                        this.api.publishExternalAccessories(settings_1.PLUGIN_NAME, [accessory]);
                       // this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;


                case 'rgb':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    const green_id=device.green_id
                    const blue_id=device.blue_id
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new RGB.RGB(this, existingAccessory,ip_address,device_name,subnet_id,device_id,channel_id,green_id,blue_id)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new RGB.RGB(this, accessory,ip_address,device_name,subnet_id,device_id,channel_id,green_id,blue_id);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;

                case 'universal_switch':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug(' universal switch uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Universal_Switch.UniversalSwitch(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Universal_Switch.UniversalSwitch(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;
                case 'security':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug('security uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Security.SecuritySensor(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id,false,0)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Security.SecuritySensor(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id,false,0);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;


                case 'speaker':
                    var UniqueID = String(subnet_id)+ '.'+ String(device_id)+ '.'+ String(channel_id)+'.'+String(device_type);
                    var uuid = this.api.hap.uuid.generate(UniqueID);
                    this.log.debug(' speaker uuid is '+uuid);
                    var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                    if (existingAccessory) {
                        this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                        new Extra.Extra(this, existingAccessory, ip_address,device_name,subnet_id,device_id,channel_id)
                    }
                    else {
                        this.log.debug('Adding new accessory:', device_name);
                        var accessory = new this.api.platformAccessory(device_name, uuid);
                        new Extra.Extra(this, accessory, ip_address,device_name,subnet_id,device_id,channel_id);

                        this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    }


                break;



            }

         }

        }
        
                    
    }

    UDPDevices(){
    
        this.server = dgram.createSocket('udp4');
    
        this.server.on('error', (err) => {
          this.log.debug(`udp server error:\n${err.stack}`);
          this.server.close();
        });
    
        this.server.on('message', (msg, rinfo) => {
          const message = Buffer.from(msg, 'hex')
            let data=message.toString('hex')
            this.DeviceStatus(data)
            //let log_packet=helper.addSpace(data)
            //this.log.debug("udp packet is received "+log_packet)
        });

        this.server.bind(6000);

    }

    DeviceStatus(data){

        let get_device_type=data.substring(42, 46)
         let raw_data=data
         //let log_packet=helper.addSpace(raw_data)
         //this.log.debug('raw packet :',log_packet);
        if(get_device_type=="0032")
        {
             let log_packet=helper.addSpace(data)
            // this.log.debug('0032 packet :',log_packet);

             let status_data=data.substring(34)
            // this.log.debug("status data "+status_data)
             let re_subnet_id=status_data.substring(0,2)
             let re_device_id=status_data.substring(2,4)
             let re_channel_id=status_data.substring(16,18)

             if(this.config.devices && this.config.devices.length !==0){

                for (const device of this.config.devices) {

                    const get_subnet_id = helper.numberToHex(device.subnet_id);
                    const get_device_id = helper.numberToHex(device.device_id);
                    const get_channel_id = helper.numberToHex(device.channel_id);
                    const get_device_type = device.device_type;
                    const ip_address= this.config.ip_address

                    const set_subnet_id = device.subnet_id;
                    const set_device_id = device.device_id;
                    const set_channel_id = device.channel_id;
                    const set_device_name = device.device_name;

            
                    if(get_subnet_id==re_subnet_id && get_device_id==re_device_id && get_channel_id===re_channel_id){
                         
                        if(get_device_type=="switch"){

                            this.log.debug("switch type matched ")
                            let re_level=status_data.substring(20,22)
                            let level =helper.HexTonumber(re_level)
                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new RelaySwitch.RelaySwitch(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value)

                            }

                        }

                        else if(get_device_type=="dimmer"){

                            this.log.debug("dimmer type matched ")
                            let re_level=status_data.substring(20,22)
                            let level =helper.HexTonumber(re_level)
                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new Relaydimmer.RelayDimmer(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value,level)

                            }

                        }

                        else if(get_device_type=="fan"){

                            this.log.debug("fan type matched ")
                            let re_level=status_data.substring(20,22)
                            let level =helper.HexTonumber(re_level)
                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new Dimmerfan.DimmerFan(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value,level)

                            }

                        }

                        else if(get_device_type=="appliances"){

                            this.log.debug("appliance type matched ")
                            let re_level=status_data.substring(20,22)
                            let level =helper.HexTonumber(re_level)
                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new Appliance.Appliance(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value)

                            }

                        }

                        else if(get_device_type=="relay_to_curtain"){

                            this.log.debug("relay to curtain type matched ")
                            this.log.debug("open relay to curtain")
                            let re_level=status_data.substring(20,22)
                            let level =helper.HexTonumber(re_level)
                             
                            let status={
                                targetposition:100
                            }

                            if(level==100){
                                var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                                var uuid = this.api.hap.uuid.generate(UniqueID);
                                var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                                if (existingAccessory) {
                                //open
                                    this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                    new RelayToCurtain.RelayToCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status,device.close_id)
                                
                            
                                }

                            }

                        }
                        else if(get_device_type=="bee_curtain"){

                            this.log.debug("bee curtain type matched ")
                            this.log.debug("open relay to curtain")
                            let re_level=status_data.substring(20,22)
                            let level =helper.HexTonumber(re_level)
                             
                            let status={
                                targetposition:level
                            }

                                var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                                var uuid = this.api.hap.uuid.generate(UniqueID);
                                var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                                if (existingAccessory) {
                                //open
                                    this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                    new BeeCurtain.BeeCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status)
                                
                            
                                }

                          

                        }


                    }
                    else if(get_subnet_id==re_subnet_id && get_device_id==re_device_id && device.close_id){

                          this.log.debug("relay to curtain type matched ")
                          this.log.debug("close relay to curtain")
                          let re_level=status_data.substring(20,22)
                          let level =helper.HexTonumber(re_level)
                           
                          let status={
                              targetposition:0
                          }

                          if(level==100){
                              var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                              var uuid = this.api.hap.uuid.generate(UniqueID);
                              var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                              if (existingAccessory) {
                              //open
                                  this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                  new RelayToCurtain.RelayToCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status,device.close_id)
                              
                          
                              }

                          }

                        


                    }

                }

             }

    
        }

        else if(get_device_type=="0034"){

            let status_data=data.substring(34)
            //this.log.debug("status 0034 data "+status_data)
            let device_command=data.substring(50)
           // this.log.debug(device_command)

            let re_subnet_id=status_data.substring(0,2)
            let re_device_id=status_data.substring(2,4)
            
            if(this.config.devices && this.config.devices.length !==0){

                for (const device of this.config.devices) {

                    const get_subnet_id = helper.numberToHex(device.subnet_id);
                    const get_device_id = helper.numberToHex(device.device_id);
                    const get_channel_id = helper.numberToHex(device.channel_id);
                    const get_device_type = device.device_type;
                    const ip_address= this.config.ip_address

                    const set_subnet_id = device.subnet_id;
                    const set_device_id = device.device_id;
                    const set_channel_id = device.channel_id;
                    const set_device_name = device.device_name;

                    if(get_subnet_id==re_subnet_id && get_device_id==re_device_id){
                   
                        //this.log.debug(device_command.substring(set_channel_id*2+0,set_channel_id*2+2))
                        let re_level=device_command.substring(set_channel_id*2+0,set_channel_id*2+2)
                        let level =helper.HexTonumber(re_level)

                        if(get_device_type=="switch"){

                            let status_value= (level > 0);
                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new RelaySwitch.RelaySwitch(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value)

                            }


                        }
                        else if(get_device_type=="dimmer"){

                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new Relaydimmer.RelayDimmer(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value,level)

                            }

                        }

                        else if(get_device_type=="fan"){

                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new Dimmerfan.DimmerFan(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value,level)

                            }
                        }

                        else if(get_device_type=="appliances"){

                            let status_value= (level > 0);

                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new Appliance.Appliance(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status_value)

                            }

                        }
                        else if(get_device_type=="relay_to_curtain"){

                            this.log.debug("relay to curtain type matched ")
                            this.log.debug("open relay to curtain")
                            let status={
                                targetposition:100
                            }

                            if(level==100){

                                var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                                var uuid = this.api.hap.uuid.generate(UniqueID);
                                var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                                if (existingAccessory) {
                                //open
                                    this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                    new RelayToCurtain.RelayToCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status,device.close_id)
                                
                            
                                }

                            }


                        }
                        else if(get_device_type=="bee_curtain"){

                            this.log.debug("bee_curtain type matched 0034 ")
                            
                            let status={
                                targetposition:level
                            }

                                var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                                var uuid = this.api.hap.uuid.generate(UniqueID);
                                var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                                if (existingAccessory) {
                                //open
                                    this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                    new BeeCurtain.BeeCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status)
                                
                            
                                }

                          


                        }
                        

                   
                    }

                    if(get_subnet_id==re_subnet_id && get_device_id==re_device_id && device.close_id){
                        const set_close_id = device.close_id;

                        let re_level=device_command.substring(set_close_id*2+0,set_close_id*2+2)
                        let level =helper.HexTonumber(re_level)

                        if(get_device_type=="relay_to_curtain"){

                            this.log.debug("relay to curtain type matched ")
                            this.log.debug("packet from 0034 close relay to curtain ")
                            let status={
                                targetposition:0
                            }

                            if(level==100){

                                var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                                var uuid = this.api.hap.uuid.generate(UniqueID);
                                var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                                if (existingAccessory) {
                                //open
                                    this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                    new RelayToCurtain.RelayToCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status,device.close_id)
                                
                            
                                }

                            }


                        }


                    }


                    


                }
            
            
    
            }

            

        }

         else if(get_device_type=="1945"){

            let log_packet=helper.addSpace(data)
            this.log.debug('floor heater packet :',log_packet);

            let status_data=data.substring(34)
            //this.log.debug("status 0034 data "+status_data)
            let device_command=data.substring(34)
           //this.log.debug(device_command)

            let re_subnet_id=device_command.substring(0,2)
            let re_device_id=device_command.substring(2,4)
            let re_channel_id=device_command.substring(16,18)

            if(this.config.devices && this.config.devices.length !==0){

                for (const device of this.config.devices) {

                    const get_subnet_id = helper.numberToHex(device.subnet_id);
                    const get_device_id = helper.numberToHex(device.device_id);
                    const get_channel_id = helper.numberToHex(device.channel_id);
                    const get_device_type = device.device_type;
                    const ip_address= this.config.ip_address

                    const set_subnet_id = device.subnet_id;
                    const set_device_id = device.device_id;
                    const set_channel_id = device.channel_id;
                    const set_device_name = device.device_name;


                    if(get_device_type=="floor_heater"){

                       let fh_status={
                        On: true,
                        FHStatus:0,
                        FHAmbient : 33,
                        FHSetpoint : 16,
                        Scale:0

                        }
                       
                        let mod_set_channel_id=set_channel_id-1
                        re_channel_id=helper.HexTonumber(re_channel_id)

                        if(re_subnet_id==get_subnet_id && get_device_id==re_device_id && mod_set_channel_id==re_channel_id){
                        
                             this.log.debug("floor Heater matched")
                             fh_status.FHAmbient=helper.HexTonumber(device_command.substring(20,22))
                             fh_status.FHSetpoint=helper.HexTonumber(device_command.substring(26,28))
                             let level=helper.HexTonumber(device_command.substring(22,24))
                             let status_value= (level > 0);
                             fh_status.On=status_value
                             fh_status.FHStatus=helper.HexTonumber(device_command.substring(22,24))
                             fh_status.Scale=helper.HexTonumber(device_command.substring(24,26))

                             this.log.debug("floor heater current temp "+fh_status.FHAmbient)
                            
                            var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                            var uuid = this.api.hap.uuid.generate(UniqueID);
                            var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                            if (existingAccessory) {

                                this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                                new FloorHeater.FloorHeater(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,fh_status,device.scale);

                            }
                       }
    



                    }



                }

            }

         }else if(get_device_type=="e3d9"){
        
            let log_packet=helper.addSpace(data)
            this.log.debug('floor heater e3d9 packet :',log_packet);
            let status_data=data.substring(34)
            let device_command=data.substring(34)
            let re_subnet_id=device_command.substring(0,2)
            let re_device_id=device_command.substring(2,4)
            let fh_no="00";
            if(this.config.devices && this.config.devices.length !==0){
                const ip_address= this.config.ip_address;
                let subnet_id=re_subnet_id;
                let device_id=re_device_id;
                let set_packet=String("0c01fefffe1944")+subnet_id+device_id+fh_no;
                let payload=set_packet
                let get_crc=crc_calculator.payload(payload)
                let send_packet=set_packet+get_crc
                let packet=helper.fullPacket(send_packet)
                let log_packet=helper.addSpace(packet)
                this.log.debug('FH  :',log_packet);
                send_to_server.udpclient(ip_address,packet)

            }



         }

         else if(get_device_type=="e0ed" || get_device_type=="e0ef"){

            let log_packet=helper.addSpace(data)
           // this.log.debug('E0ED and e0ef packet :',log_packet);

            let status_data=data.substring(34)
            //this.log.debug("status 0034 data "+status_data)
            let device_command=data.substring(34)
           //this.log.debug(device_command)

            let re_subnet_id=device_command.substring(0,2)
            let re_device_id=device_command.substring(2,4)
            let re_channel_id=device_command.substring(18,20)


            if(this.config.devices && this.config.devices.length !==0){

                for (const device of this.config.devices) {

                    const get_subnet_id = helper.numberToHex(device.subnet_id);
                    const get_device_id = helper.numberToHex(device.device_id);
                    const get_channel_id = helper.numberToHex(device.channel_id);
                    const get_device_type = device.device_type;
                    const ip_address= this.config.ip_address

                    const set_subnet_id = device.subnet_id;
                    const set_device_id = device.device_id;
                    const set_channel_id = device.channel_id;
                    const set_device_name = device.device_name;


                    if(get_device_type=="thermostat"){

                       let th_status={
                        On: true,
                        THAmbient : 28,
                        THSetpoint : 16,
                        acstatus:1,
                        autoTemperature:14,
                        coolTemperature:14,
                        dryTemperature:14,
                        heatTemperature:14,
                        currentmode:1,
                        fanmode:3,
                        Scale:0,
                        setting_mode:"02"

                        }
                       
                        let mod_set_channel_id=set_channel_id-1
                        re_channel_id=helper.HexTonumber(re_channel_id)

                        if(re_subnet_id==get_subnet_id && get_device_id==re_device_id && mod_set_channel_id==re_channel_id){
                        
                             this.log.debug(" thermostat matched")
                             let get_current_mode=5

                             let level=helper.HexTonumber(device_command.substring(20,22))
                             let status_value= (level > 0);
                             th_status.On=status_value
                             th_status.acstatus=helper.HexTonumber(device_command.substring(20,22))

                            
                             th_status.coolTemperature=helper.HexTonumber(device_command.substring(22,24))
                             let get_current_mode_status=helper.HexTonumber(device_command.substring(24,25))

                             if(status_value){

                                get_current_mode=helper.HexTonumber(device_command.substring(24,25))
                             }
                            

                             th_status.currentmode=helper.current_mode(get_current_mode)

                             if(get_current_mode_status==0){

                                th_status.THSetpoint=helper.HexTonumber(device_command.substring(22,24))
                             }else  if(get_current_mode_status==1){
                                
                                th_status.THSetpoint=helper.HexTonumber(device_command.substring(30,32))

                             }
                             else  if(get_current_mode_status==3){
                                
                                th_status.THSetpoint=helper.HexTonumber(device_command.substring(34,36))

                             }

                             else  if(get_current_mode_status==2){
                                
                                th_status.THSetpoint=helper.HexTonumber(24)

                             }

                             th_status.setting_mode=device_command.substring(24,26)
                             
                             th_status.fanmode=helper.HexTonumber(device_command.substring(25,26))
                             th_status.THAmbient=helper.HexTonumber(device_command.substring(28,30))
                             if(th_status.THAmbient>=100){
                                th_status.THAmbient=100
                             }
                             th_status.heatTemperature=helper.HexTonumber(device_command.substring(30,32))
                             th_status.autoTemperature=helper.HexTonumber(device_command.substring(34,36))
                             th_status.dryTemperature=helper.HexTonumber(device_command.substring(36,38))

                        var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                        var uuid = this.api.hap.uuid.generate(UniqueID);
                        var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                        if (existingAccessory) {

                            this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                             new Thermostat.Thermostat(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,th_status,device.scale);

                        }
                       }
    

                       
                    }



                }

            }

         }
          else if(get_device_type=="e3e1"){

           this.log.debug("curatain motor matched")

           let log_packet=helper.addSpace(data)
           // this.log.debug('E3E1 packet :',log_packet);
            let status_data=data.substring(34)
            
            let device_command=data.substring(34)
           //this.log.debug(device_command)

            let re_subnet_id=device_command.substring(0,2)
            let re_device_id=device_command.substring(2,4)
            let re_channel_status=device_command.substring(18,20)


            if(this.config.devices && this.config.devices.length !==0){

                for (const device of this.config.devices) {

                    const get_subnet_id = helper.numberToHex(device.subnet_id);
                    const get_device_id = helper.numberToHex(device.device_id);
                    const get_channel_id = helper.numberToHex(device.channel_id);
                    const get_device_type = device.device_type;
                    const ip_address= this.config.ip_address

                    const set_subnet_id = device.subnet_id;
                    const set_device_id = device.device_id;
                    const set_channel_id = device.channel_id;
                    const set_device_name = device.device_name;
           
                     if(re_subnet_id==get_subnet_id && get_device_id==re_device_id){
                    
                     //this.log.debug(re_channel_status);

                     let targetposition=0
                      
                     re_channel_status=re_channel_status.toString()

                     
                      if(re_channel_status=="01" && device.send_option=="reversely"){
                        targetposition=0
                        //this.log.debug("01 matched close reversely")
                      }
                      else if(re_channel_status=="02" && device.send_option=="reversely"){
                        targetposition=100

                        //this.log.debug("02 matched open reversely")
                       
                      }
                      else if(re_channel_status=="01" && device.send_option=="actually"){
                       
                        targetposition=0
                       // this.log.debug("01 matched close actually")

                      }
                      else if(re_channel_status=="02" && device.send_option=="actually"){
                        targetposition=100
                       // this.log.debug("02 matched open actually")
                        
                      }else{

                        this.log.debug("nothing matched")
                      }
      

                     let status={
                        targetposition:targetposition
                     }

                        var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                        var uuid = this.api.hap.uuid.generate(UniqueID);
                        var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                        if (existingAccessory) {

                            this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                             new RelayToCurtain.RelayToCurtain(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,status,device.send_option);

                        }
                         
                    
                    
                    }

                









                }
            
            }


         }

         else if(get_device_type=="012d"){

            let log_packet=helper.addSpace(data)
            this.log.debug('packet 012d:',log_packet);

            let status_data=data.substring(34)
            let re_subnet_id=status_data.substring(0,2)
            let re_device_id=status_data.substring(2,4)
            let command=status_data.substring(146)

            this.log.debug("subnet id ",re_subnet_id)
            this.log.debug("device  id ",re_device_id)


            

            //let re_level=device_command.substring(set_close_id*2+0,set_close_id*2+2)

            if(this.config.devices && this.config.devices.length !==0){

               for (const device of this.config.devices) {

                   const get_subnet_id = helper.numberToHex(device.subnet_id);
                   const get_device_id = helper.numberToHex(device.device_id);
                   const get_channel_id = helper.numberToHex(device.channel_id);
                   const get_device_type = device.device_type;
                   const ip_address= this.config.ip_address

                   const set_subnet_id = device.subnet_id;
                   const set_device_id = device.device_id;
                   const set_channel_id = device.channel_id;
                   const set_device_name = device.device_name;

                   if(re_subnet_id==get_subnet_id && get_device_id==re_device_id){

                       
                       let re_level=command.substring(set_channel_id*2+0,set_channel_id*2+2)

                          let data=0

                          this.log.debug("status ",typeof re_level)
                        if(re_level=="01"){
                           data=3
                          }

                       var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                       var uuid = this.api.hap.uuid.generate(UniqueID);
                       var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                       if (existingAccessory) {

                           this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                            new Security.SecuritySensor(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,data);

                       }

                   }


               }
           }


        }

        //
        else if(get_device_type=="011f" || get_device_type== "0105"){

            let log_packet=helper.addSpace(data)
            this.log.debug('packet 011f:',log_packet);

            let status_data=data.substring(34)
            console.log(" command "+status_data)
            let re_subnet_id=status_data.substring(0,2)
            let re_device_id=status_data.substring(2,4)
            let re_channel_id=status_data.substring(16,18)

            if(this.config.devices && this.config.devices.length !==0){

               for (const device of this.config.devices) {

                   const get_subnet_id = helper.numberToHex(device.subnet_id);
                   const get_device_id = helper.numberToHex(device.device_id);
                   const get_channel_id = helper.numberToHex(device.channel_id);
                   const get_device_type = device.device_type;
                   const ip_address= this.config.ip_address

                   const set_subnet_id = device.subnet_id;
                   const set_device_id = device.device_id;
                   const set_channel_id = device.channel_id;
                   const set_device_name = device.device_name;

                   if(re_subnet_id==get_subnet_id && get_device_id==re_device_id && re_channel_id==get_channel_id){

                       let re_level=status_data.substring(18,20)

                       let data=helper.current_mode_security(helper.HexTonumber(re_level))
                       

                       var UniqueID = String(set_subnet_id)+ '.'+ String(set_device_id)+ '.'+ String(set_channel_id)+'.'+String(get_device_type);
                       var uuid = this.api.hap.uuid.generate(UniqueID);
                       var existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
                       if (existingAccessory) {

                           this.log.debug('Restoring existing accessory from cache:', existingAccessory.displayName);
                            new Security.SecuritySensor(this, existingAccessory, ip_address,set_device_name,set_subnet_id,set_device_id,set_channel_id,true,data);

                       }

                   }


               }
           }


        }



    }

    

 }



           
exports.TISHomebridge = TISHomebridge;