const crc_calculator = require("./crc/crc16");
const  send_to_server = require("./server");
const  helper = require("./helper");

exports.SecuritySensor = void 0;

class SecuritySensor {
    constructor(platform, accessory,ip_address, device_name,subnet_id,device_id,channel_id,value,status) {
        this.platform = platform;
        this.accessory = accessory;
        this.device_name = device_name;
        this.subnet_id=subnet_id;
        this.device_id=device_id;
        this.channel_id=channel_id;
        this.ip_address=ip_address,
        this.currentState=status

        if(value==false){

            this.accessory.getService(this.platform.Service.AccessoryInformation)
                .setCharacteristic(this.platform.Characteristic.Manufacturer, 'TIS Control')
                .setCharacteristic(this.platform.Characteristic.Manufacturer.Model, "Security Sensor");
            this.service = this.accessory.getService(this.platform.Service.SecuritySystem) || this.accessory.addService(this.platform.Service.SecuritySystem);
            this.service.setCharacteristic(this.platform.Characteristic.Name, device_name);
            this.service.getCharacteristic(this.platform.Characteristic.SecuritySystemCurrentState)
                .onGet(this.getCurrentStatus.bind(this));

            this.service.getCharacteristic(this.platform.Characteristic.SecuritySystemTargetState)
                .onSet(this.setOn.bind(this))
                .onGet(this.getOn.bind(this));

        }else{
           
           this.service = this.accessory.getService(this.platform.Service.SecuritySystem) || this.accessory.addService(this.platform.Service.SecuritySystem);
           this.service.getCharacteristic(this.platform.Characteristic.SecuritySystemCurrentState)
           .onGet(this.getCurrentStatus.bind(this)).updateValue(this.currentState);

           this.service.getCharacteristic(this.platform.Characteristic.SecuritySystemTargetState)
           .onSet(this.setOn.bind(this)).updateValue(this.currentState);


       }
  
    }
    async setOn(value) {
       
        this.platform.log.debug(" sensor send "+value)
        this.currentState=value
        let mode=helper.current_mode_security_native(value)

        let subnet_id=helper.numberToHex(this.subnet_id )
        let device_id=helper.numberToHex(this.device_id ) 
        let channel_id=helper.numberToHex(this.channel_id) 
        let set_packet=String("0d01fefffe0104")+subnet_id+device_id+channel_id+helper.numberToHex(mode) 
        let payload=set_packet
        let get_crc=crc_calculator.payload(payload)
        let send_packet=set_packet+get_crc
        let packet=helper.fullPacket(send_packet)
        let log_packet=helper.addSpace(packet)
        this.platform.log.debug(log_packet)
        send_to_server.udpclient(this.ip_address,packet)
    }
    async getOn() {
        this.platform.log.debug(" security panel status ")
         //let valid=helper.functioncheck()
         //this.platform.log.debug(valid)

         let subnet_id=helper.numberToHex(this.subnet_id )
         let device_id=helper.numberToHex(this.device_id ) 
         let channel_id=helper.numberToHex(this.channel_id) 
         let set_packet=String("0c01fefffe011e")+subnet_id+device_id+channel_id
         let payload=set_packet
         let get_crc=crc_calculator.payload(payload)
         let send_packet=set_packet+get_crc
         let packet=helper.fullPacket(send_packet)
         let log_packet=helper.addSpace(packet)
         this.platform.log.debug(log_packet)
         send_to_server.udpclient(this.ip_address,packet)

        return  this.currentState;

    }

    async getCurrentStatus() {
        this.platform.log.debug("security panel status ")
         //let valid=helper.functioncheck()
         //this.platform.log.debug(valid)

         let subnet_id=helper.numberToHex(this.subnet_id )
         let device_id=helper.numberToHex(this.device_id ) 
         let channel_id=helper.numberToHex(this.channel_id) 
         let set_packet=String("0c01fefffe001e")+subnet_id+device_id+channel_id
         let payload=set_packet
         let get_crc=crc_calculator.payload(payload)
         let send_packet=set_packet+get_crc
         let packet=helper.fullPacket(send_packet)
         let log_packet=helper.addSpace(packet)
         this.platform.log.debug(log_packet)
         send_to_server.udpclient(this.ip_address,packet)

        return  this.currentState;
    }


}
exports.SecuritySensor = SecuritySensor;